@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2017
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -954,8 +954,8 @@ void Parser::Parse( const vector<Word*>& words,
 		    TimerBlock& timers ){
   timers.parseTimer.start();
   if ( !isInit ){
-    LOG << "Parser is not initialized!" << endl;
-    exit(1);
+    LOG << "Parser is not initialized! EXIT!" << endl;
+    throw runtime_error( "Parser is not initialized!" );
   }
   if ( words.empty() ){
     LOG << "unable to parse an analisis without words" << endl;
