@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2017
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -48,10 +48,16 @@ using namespace std;
 using namespace TiCC;
 using namespace folia;
 
+#define LOG *Log(myLog)
+
 string toString( const Compound::Type& ct ){
   switch ( ct ){
   case Compound::Type::NN:
     return "NN";
+  case Compound::Type::NNN:
+    return "NNN";
+  case Compound::Type::NVN:
+    return "NVN";
   case Compound::Type::NA:
     return "NA";
   case Compound::Type::NB:
@@ -106,6 +112,96 @@ string toString( const Compound::Type& ct ){
   return "DEADLY";
 }
 
+Compound::Type stringToCompound( const string& s ){
+  if ( s == "NN" ){
+    return Compound::Type::NN;
+  }
+  else if ( s == "NNN" ){
+    return Compound::Type::NNN;
+  }
+  else if ( s == "NVN" ){
+    return Compound::Type::NVN;
+  }
+  else if ( s == "NA" ){
+    return Compound::Type::NA;
+  }
+  else if ( s == "NB" ){
+    return Compound::Type::NB;
+  }
+  else if ( s == "NP" ){
+    return Compound::Type::NP;
+  }
+  else if ( s == "NV" ){
+    return  Compound::Type::NV;
+  }
+  else if ( s == "AN" ){
+    return  Compound::Type::AN;
+  }
+  else if ( s == "AA" ){
+    return  Compound::Type::AA;
+  }
+  else if ( s == "AB" ){
+    return  Compound::Type::AB;
+  }
+  else if ( s == "AP" ){
+    return  Compound::Type::AP;
+  }
+  else if ( s == "AV" ){
+    return  Compound::Type::AV;
+  }
+  else if ( s == "BN" ){
+    return  Compound::Type::BN;
+  }
+  else if ( s == "BA" ){
+    return  Compound::Type::BA;
+  }
+  else if ( s == "BB" ){
+    return  Compound::Type::BB;
+  }
+  else if ( s == "BP" ){
+    return  Compound::Type::BP;
+  }
+  else if ( s == "BV" ){
+    return  Compound::Type::BV;
+  }
+  else if ( s == "PN" ){
+    return  Compound::Type::PN;
+  }
+  else if ( s == "PA" ){
+    return  Compound::Type::PA;
+  }
+  else if ( s == "PB" ){
+    return  Compound::Type::PB;
+  }
+  else if ( s == "PP" ){
+    return  Compound::Type::PP;
+  }
+  else if ( s == "PV" ){
+    return  Compound::Type::PV;
+  }
+  else if ( s == "VN" ){
+    return  Compound::Type::VN;
+  }
+  else if ( s == "VA" ){
+    return  Compound::Type::VA;
+  }
+  else if ( s == "VB" ){
+    return  Compound::Type::VB;
+  }
+  else if ( s == "VP" ){
+    return  Compound::Type::VP;
+  }
+  else if ( s == "VV" ){
+    return  Compound::Type::VV;
+  }
+  else if ( s.empty() || s == "none" ){
+    return  Compound::Type::NONE;
+  }
+  else {
+    throw runtime_error( "no such compound:" + s );
+  }
+}
+
 ostream& operator<<( ostream& os, const Compound::Type& ct ){
   os << toString( ct );
   return os;
@@ -139,8 +235,9 @@ ostream& operator<<( ostream& os, const Status& st ){
   return os;
 }
 
-BracketLeaf::BracketLeaf( const RulePart& p, int flag ):
-  BaseBracket(p.ResultClass, p.RightHand, flag),
+BracketLeaf::BracketLeaf( const RulePart& p, int flag, LogStream& l ):
+  BaseBracket(p.ResultClass, p.RightHand, flag, l ),
+  glue(false),
   morph(p.morpheme )
 {
   ifpos = -1;
@@ -169,6 +266,7 @@ BracketLeaf::BracketLeaf( const RulePart& p, int flag ):
   else {
     orig = toString( cls );
     orig += "_";
+    glue = p.is_glue;
     for ( size_t i = 0; i < RightHand.size(); ++i ){
       orig += toString(RightHand[i]);
       if ( RightHand[i] == CLEX::AFFIX ){
@@ -184,8 +282,11 @@ BracketLeaf::BracketLeaf( const RulePart& p, int flag ):
   }
 }
 
-BracketLeaf::BracketLeaf( CLEX::Type t, const UnicodeString& us, int flag ):
-  BaseBracket( t, vector<CLEX::Type>(), flag ),
+BracketLeaf::BracketLeaf( CLEX::Type t,
+			  const UnicodeString& us,
+			  int flag,
+			  LogStream& l ):
+  BaseBracket( t, vector<CLEX::Type>(), flag, l ),
   morph( us )
 {
   ifpos = -1;
@@ -195,8 +296,9 @@ BracketLeaf::BracketLeaf( CLEX::Type t, const UnicodeString& us, int flag ):
 
 BracketNest::BracketNest( CLEX::Type t,
 			  Compound::Type c,
-			  int flag ): BaseBracket( t, flag ),
-				      _compound( c )
+			  int flag,
+			  LogStream& l ): BaseBracket( t, flag, l ),
+					  _compound( c )
 {
   _status = Status::COMPLEX;
 }
@@ -286,19 +388,19 @@ bool BracketNest::testMatch( list<BaseBracket*>& result,
 			     const list<BaseBracket*>::iterator& rpos,
 			     list<BaseBracket*>::iterator& bpos ){
   if ( debugFlag > 5 ){
-    cerr << "test MATCH " << endl;
+    LOG << "test MATCH: rpos= " << *rpos << endl;
   }
   bpos = result.end();
   size_t len = (*rpos)->RightHand.size();
   if ( len == 0 || len > result.size() ){
     if ( debugFlag > 5 ){
-      cerr << "test MATCH FAIL (no RHS or RHS > result)" << endl;
+      LOG << "test MATCH FAIL (no RHS or RHS > result)" << endl;
     }
     return false;
   }
   size_t fpos = (*rpos)->infixpos();
   if ( debugFlag > 5 ){
-    cerr << "test MATCH, fpos=" << fpos << " en len=" << len << endl;
+    LOG << "test MATCH, fpos=" << fpos << " en len=" << len << endl;
   }
   list<BaseBracket*>::iterator it = rpos;
   while ( fpos > 0 ){
@@ -309,18 +411,18 @@ bool BracketNest::testMatch( list<BaseBracket*>& result,
   bpos = it;
   for (; j < len && it != result.end(); ++j, ++it ){
     if ( debugFlag > 5 ){
-      cerr << "test MATCH vergelijk " << *it << " met " << (*rpos)->RightHand[j] << endl;
+      LOG << "test MATCH vergelijk " << *it << " met " << (*rpos)->RightHand[j] << endl;
     }
-    if ( (*rpos)->RightHand[j] == CLEX::XAFFIX){
+    if ( (*rpos)->RightHand[j] == CLEX::XAFFIX ){
       continue;
     }
-    else if ( (*rpos)->RightHand[j] == CLEX::AFFIX){
+    else if ( (*rpos)->RightHand[j] == CLEX::AFFIX ){
       continue;
     }
     else if ( (*rpos)->RightHand[j] != (*it)->tag() ){
       if ( debugFlag > 5 ){
-	cerr << "test MATCH FAIL (" << (*rpos)->RightHand[j]
-	     << " != " << (*it)->tag() << ")" << endl;
+	LOG << "test MATCH FAIL (" << (*rpos)->RightHand[j]
+	    << " != " << (*it)->tag() << ")" << endl;
       }
       (*rpos)->set_status(Status::FAILED);
       bpos = it;
@@ -329,110 +431,41 @@ bool BracketNest::testMatch( list<BaseBracket*>& result,
   }
   if ( j < len ){
     if ( debugFlag > 5 ){
-      cerr << "test MATCH FAIL (j < len)" << endl;
+      LOG << "test MATCH FAIL (j < len)" << endl;
     }
     bpos = result.end();
     return false;
   }
   if ( debugFlag > 5 ){
-    cerr << "test MATCH OK" << endl;
+    LOG << "test MATCH OK" << endl;
   }
   return true;
 }
 
-Compound::Type construct( const CLEX::Type tag1, const CLEX::Type tag2 ){
-  Compound::Type compound = Compound::Type::NONE;
-  switch ( tag1 ){
-  case CLEX::N:
-    switch( tag2 ){
-    case CLEX::N: compound = Compound::Type::NN;
-      break;
-    case CLEX::A: compound = Compound::Type::NA;
-      break;
-    case CLEX::B: compound = Compound::Type::NB;
-      break;
-    case CLEX::P: compound = Compound::Type::NP;
-      break;
-    case CLEX::V: compound = Compound::Type::NV;
-      break;
-    default:
-      break;
-    }
-    break;
-  case CLEX::A:
-    switch( tag2 ){
-    case CLEX::N: compound = Compound::Type::AN;
-      break;
-    case CLEX::A: compound = Compound::Type::AA;
-      break;
-    case CLEX::B: compound = Compound::Type::AB;
-      break;
-    case CLEX::P: compound = Compound::Type::AP;
-      break;
-    case CLEX::V: compound = Compound::Type::AV;
-      break;
-    default:
-      break;
-    }
-    break;
-  case CLEX::B:
-    switch( tag2 ){
-    case CLEX::N: compound = Compound::Type::BN;
-      break;
-    case CLEX::A: compound = Compound::Type::BA;
-      break;
-    case CLEX::B: compound = Compound::Type::BB;
-      break;
-    case CLEX::P: compound = Compound::Type::BP;
-      break;
-    case CLEX::V: compound = Compound::Type::BV;
-      break;
-    default:
-      break;
-    }
-    break;
-  case CLEX::P:
-    switch( tag2 ){
-    case CLEX::N: compound = Compound::Type::PN;
-      break;
-    case CLEX::A: compound = Compound::Type::PA;
-      break;
-    case CLEX::B: compound = Compound::Type::PB;
-      break;
-    case CLEX::P: compound = Compound::Type::PP;
-      break;
-    case CLEX::V: compound = Compound::Type::PV;
-      break;
-    default:
-      break;
-    }
-    break;
-  case CLEX::V:
-    switch( tag2 ){
-    case CLEX::N: compound = Compound::Type::VN;
-      break;
-    case CLEX::A: compound = Compound::Type::VA;
-      break;
-    case CLEX::B: compound = Compound::Type::VB;
-      break;
-    case CLEX::P: compound = Compound::Type::VP;
-      break;
-    case CLEX::V: compound = Compound::Type::VV;
-      break;
-    default:
-      break;
-    }
-    break;
-  default:
-    break;
+Compound::Type construct( const vector<CLEX::Type> tags ){
+  string s;
+  for ( const auto& t : tags ){
+    s += toString( t );
   }
-  return compound;
+  try {
+    return stringToCompound( s );
+  }
+  catch (...){
+    return Compound::Type::NONE;
+  }
+}
+
+Compound::Type construct( const CLEX::Type tag1, const CLEX::Type tag2 ){
+  vector<CLEX::Type> v;
+  v.push_back( tag1 );
+  v.push_back( tag2 );
+  return construct( v );
 }
 
 Compound::Type BracketNest::getCompoundType(){
   if ( debugFlag > 5 ){
-    cerr << "get compoundType: " << this << endl;
-    cerr << "#parts: " << parts.size() << endl;
+    LOG << "get compoundType: " << this << endl;
+    LOG << "#parts: " << parts.size() << endl;
   }
   Compound::Type compound = Compound::Type::NONE;
   if ( parts.size() == 1 ){
@@ -448,12 +481,13 @@ Compound::Type BracketNest::getCompoundType(){
     Compound::Type cp2 = (*it)->compound();
     Status st2 = (*it)->status();
     if ( debugFlag > 5 ){
-      cerr << "tag1 :" << tag1 << " stat1: " << st1 << " cp1: " << cp1 << endl;
-      cerr << "tag2 :" << tag2 << " stat2: " << st2 << " cp2: " << cp2 << endl;
+      LOG << "tag1 :" << tag1 << " stat1: " << st1 << " cp1: " << cp1 << endl;
+      LOG << "tag2 :" << tag2 << " stat2: " << st2 << " cp2: " << cp2 << endl;
     }
     if ( st1 != Status::FAILED
 	 && st2 != Status::FAILED ){
       if ( tag1 == CLEX::N
+	   && st1 != Status::DERIVATIONAL
 	   && st1 != Status::PARTICLE
 	   && st1 != Status::PARTICIPLE ){
 	if ( st2 == Status::STEM ){
@@ -511,14 +545,16 @@ Compound::Type BracketNest::getCompoundType(){
     Compound::Type cp1 = (*it)->compound();
     CLEX::Type tag1 = (*it)->tag();
     Status st1 = (*it)->status();
-    CLEX::Type tag2 = (*++it)->tag();
+    Compound::Type cp2 = (*++it)->compound();
+    CLEX::Type tag2 = (*it)->tag();
     Status st2 = (*it)->status();
-    CLEX::Type tag3 = (*++it)->tag();
+    Compound::Type cp3 = (*++it)->compound();
+    CLEX::Type tag3 = (*it)->tag();
     Status st3 = (*it)->status();
     if ( debugFlag > 5 ){
-      cerr << "tag1 :" << tag1 << " stat1: " << st1 << endl;
-      cerr << "tag2 :" << tag2 << " stat2: " << st2 << endl;
-      cerr << "tag3 :" << tag3 << " stat3: " << st3 << endl;
+      LOG << "tag1 :" << tag1 << " stat1: " << st1 << " cp1: " << cp1 << endl;
+      LOG << "tag2 :" << tag2 << " stat2: " << st2 << " cp2: " << cp2 << endl;
+      LOG << "tag3 :" << tag3 << " stat3: " << st3 << " cp3: " << cp3 << endl;
     }
     if ( st1 != Status::FAILED
 	 && st2 != Status::FAILED
@@ -536,6 +572,10 @@ Compound::Type BracketNest::getCompoundType(){
 	       && (tag3 == CLEX::NEUTRAL || st3 == Status::INFLECTION ) ) {
 	    compound = Compound::Type::NN;
 	  }
+	  else if ( ( tag2 == CLEX::V && st2 == Status::STEM )
+		    && ( tag3 == CLEX::N && st3 == Status::STEM ) ){
+	    compound = Compound::Type::NVN;
+	  }
 	  else if ( ( tag2 == CLEX::A &&
 		      ( st2 == Status::STEM || st2 == Status::COMPLEX ) )
 		    && ( tag3 == CLEX::A && st3 == Status::DERIVATIONAL ) ){
@@ -544,15 +584,27 @@ Compound::Type BracketNest::getCompoundType(){
 	  else if ( st2 == Status::DERIVATIONAL && tag3 == CLEX::NEUTRAL ){
 	    compound = cp1;
 	  }
-	  else if ( st2 == Status::INFLECTION && tag3 == CLEX::NEUTRAL ){
+	  else if ( st2 == Status::INFLECTION &&
+		    ( tag3 == CLEX::NEUTRAL || st3 == Status::INFLECTION ) ){
 	    compound = cp1;
 	  }
-	  else if ( st2 == Status::DERIVATIONAL && tag3 == CLEX::N ){
+	  else if (  st2 == Status::DERIVATIONAL
+		     && tag3 == CLEX::N ){
+	    if  ( cp3 == Compound::Type::NN ||
+		  cp3 == Compound::Type::NNN ) {
+	      compound = Compound::Type::NNN;
+	    }
+	    else {
+	      compound = Compound::Type::NN;
+	    }
+	  }
+	  else if ( st3 == Status::DERIVATIONAL
+		    && tag3 == CLEX::N ){
 	    compound = Compound::Type::NN;
 	  }
 	}
 	else if ( tag2 == CLEX::N && tag3 == CLEX::N ){
-	  compound = Compound::Type::NN;
+	  compound = Compound::Type::NNN;
 	}
       }
       else if ( tag1 == CLEX::A ){
@@ -579,6 +631,9 @@ Compound::Type BracketNest::getCompoundType(){
 	     ( st3 == Status::INFLECTION || tag3 == CLEX::NEUTRAL ) ){
 	  compound = construct( tag1, tag2 );
 	}
+	else if ( st2 == Status::COMPLEX ){
+	  compound = cp2;
+	}
 	else if ( tag3 == CLEX::NEUTRAL ){
 	  compound = construct( tag1, tag2 );
 	}
@@ -587,22 +642,33 @@ Compound::Type BracketNest::getCompoundType(){
 	}
       }
       else if ( tag1 == CLEX::B && st1 == Status::STEM ){
-      	if ( (st2 == Status::STEM || st2 == Status::COMPLEX)
-	     &&
-	     ( st3 == Status::INFLECTION || tag3 == CLEX::NEUTRAL ) ){
+      	if ( ( st2 == Status::STEM
+	       && ( st3 == Status::INFLECTION || tag3 == CLEX::NEUTRAL ) ) ){
 	  compound = construct( tag1, tag2 );
 	}
+      	else if ( st2 == Status::COMPLEX ){
+	  if ( tag2 == CLEX::N ){
+	    compound = Compound::Type::BN;
+	  }
+	  else {
+	    compound = cp2;
+	  }
+	}
       }
       else if ( tag1 == CLEX::V ){
       	if ( st2 == Status::STEM &&
 	     ( st3 == Status::INFLECTION || tag3 == CLEX::NEUTRAL ) ){
 	  compound = construct( tag1, tag2 );
 	}
+      	else if ( st3 == Status::STEM &&
+	     ( st2 == Status::INFLECTION ) ){
+	  compound = construct( tag1, tag3 );
+	}
       }
     }
   }
   if ( debugFlag > 5 ){
-    cerr << "   ASSIGNED :" << compound << endl;
+    LOG << "   ASSIGNED :" << compound << endl;
   }
   _compound = compound;
   return compound;
@@ -619,10 +685,13 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
 				       int& cnt ) const {
   Morpheme *result = 0;
   desc.clear();
+  string::size_type pos = orig.find( "^" );
+  bool glue = ( pos != string::npos );
   if ( _status == Status::COMPLEX ){
     abort();
   }
-  if ( _status == Status::STEM ){
+  else if ( _status == Status::STEM
+	    || ( _status == Status::DERIVATIONAL && glue ) ){
     KWargs args;
     args["set"] = Mbma::mbma_tagset;
     args["class"] = "stem";
@@ -641,14 +710,23 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
     ++cnt;
     args.clear();
     args["set"] = Mbma::clex_tagset;
-    args["class"] = toString( tag() );
+    if ( glue ){
+      string tag;
+      tag += orig[pos+1];
+      args["class"] = tag;
+      desc = "[" + out + "]" + CLEX::get_tDescr( CLEX::toCLEX(tag) ); // spread the word upwards!
+
+    }
+    else {
+      args["class"] = toString( tag() );
+      desc = "[" + out + "]" + CLEX::get_tDescr( tag() ); // spread the word upwards!
+    }
 #pragma omp critical(foliaupdate)
     {
       result->addPosAnnotation( args );
     }
-    desc = "[" + out + "]" + CLEX::get_tDescr( tag() ); // spread the word upwards!
   }
-  if ( _status == Status::PARTICLE ){
+  else if ( _status == Status::PARTICLE ){
     KWargs args;
     args["set"] = Mbma::mbma_tagset;
     args["class"] = "particle";
@@ -676,7 +754,7 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
   }
   else if ( _status == Status::INFLECTION ){
     KWargs args;
-    args["class"] = "affix";
+    args["class"] = "inflection";
     args["set"] = Mbma::mbma_tagset;
     result = new Morpheme( args, doc );
     args.clear();
@@ -719,13 +797,21 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
 	    || _status == Status::PARTICIPLE
 	    || _status == Status::FAILED ){
     KWargs args;
-    args["class"] = "derivational";
+    if ( _status == Status::DERIVATIONAL ){
+      args["class"] = "affix";
+    }
+    else if ( _status == Status::PARTICIPLE ){
+      args["class"] = "participle";
+    }
+    else {
+      args["class"] = "derivational";
+    }
     args["set"] = Mbma::mbma_tagset;
     result = new Morpheme( args, doc );
     args.clear();
     string out = UnicodeToUTF8(morph);
     if ( out.empty() ){
-      cerr << "problem: " << this << endl;
+      LOG << "problem: " << this << endl;
       throw logic_error( "Derivation with empty morpheme" );
     }
     args["value"] = out;
@@ -736,8 +822,6 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
     }
     ++cnt;
     desc = "[" + out + "]"; // pass it up!
-    // if ( _status == Status::PARTICIPLE )
-    //   desc += "part";
     for ( const auto& inf : inflect ){
       if ( inf != '/' ){
 	string d = CLEX::get_iDescr( inf );
@@ -756,12 +840,12 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
       result->append( feat );
     }
     args.clear();
-    args["set"] = Mbma::clex_tagset;
-    args["class"] = orig;
-#pragma omp critical(foliaupdate)
-    {
-      result->addPosAnnotation( args );
-    }
+//     args["set"] = Mbma::clex_tagset;
+//     args["class"] = orig;
+// #pragma omp critical(foliaupdate)
+//     {
+//       result->addPosAnnotation( args );
+//     }
   }
   else if ( _status == Status::INFO ){
     KWargs args;
@@ -812,6 +896,19 @@ Morpheme *BracketNest::createMorpheme( Document *doc,
     Morpheme *m = it->createMorpheme( doc,
 				      deeper_desc,
 				      deep_cnt );
+    if ( it->status() == Status::DERIVATIONAL
+	 || it->status() == Status::PARTICIPLE ){
+      if ( !it->original().empty() ){
+	args.clear();
+	args["subset"] = "applied_rule";
+	args["class"] = it->original();
+#pragma omp critical(foliaupdate)
+	{
+	  folia::Feature *feat = new folia::Feature( args );
+	  result->append( feat );
+	}
+      }
+    }
     if ( m ){
       desc += deeper_desc;
       cnt += deep_cnt;
@@ -859,13 +956,13 @@ Morpheme *BracketNest::createMorpheme( Document *doc,
 list<BaseBracket*>::iterator BracketNest::resolveAffix( list<BaseBracket*>& result,
 							const list<BaseBracket*>::iterator& rpos ){
   if ( debugFlag > 5 ){
-    cerr << "resolve affix" << endl;
+    LOG << "resolve affix" << endl;
   }
   list<BaseBracket*>::iterator bit;
   bool matched = testMatch( result, rpos, bit );
   if ( matched ){
     if ( debugFlag > 5 ){
-      cerr << "OK een match" << endl;
+      LOG << "OK een match" << endl;
     }
     size_t len = (*rpos)->RightHand.size();
     if ( len == result.size() ){
@@ -877,16 +974,16 @@ list<BaseBracket*>::iterator BracketNest::resolveAffix( list<BaseBracket*>& resu
     else {
       list<BaseBracket*>::iterator it = bit--;
       BracketNest *tmp
-	= new BracketNest( (*rpos)->tag(), Compound::Type::NONE, debugFlag );
+	= new BracketNest( (*rpos)->tag(), Compound::Type::NONE, debugFlag, myLog );
       for ( size_t j = 0; j < len; ++j ){
 	tmp->append( *it );
 	if ( debugFlag > 5 ){
-	  cerr << "erase " << *it << endl;
+	  LOG << "erase " << *it << endl;
 	}
 	it = result.erase(it);
       }
       if ( debugFlag > 5 ){
-	cerr << "new node:" << tmp << endl;
+	LOG << "new node:" << tmp << endl;
       }
       _compound = tmp->getCompoundType();
       result.insert( ++bit, tmp );
@@ -903,7 +1000,7 @@ list<BaseBracket*>::iterator BracketNest::resolveAffix( list<BaseBracket*>& resu
 
 void BracketNest::resolveNouns( ){
   if ( debugFlag > 5 ){
-    cerr << "resolve NOUNS in:" << this << endl;
+    LOG << "resolve NOUNS in:" << this << endl;
   }
   list<BaseBracket*>::iterator it = parts.begin();
   list<BaseBracket*>::iterator prev = it++;
@@ -911,23 +1008,27 @@ void BracketNest::resolveNouns( ){
     if ( (*prev)->tag() == CLEX::N && (*prev)->RightHand.size() == 0
 	 && ( (*it)->tag() == CLEX::N && (*it)->status() == Status::STEM )
 	 && (*it)->RightHand.size() == 0 ){
-      BaseBracket *tmp
-	= new BracketNest( CLEX::N, Compound::Type::NN, debugFlag );
+      Compound::Type newt = Compound::Type::NN;
+      if ( (*prev)->compound() == Compound::Type::NN
+	   || (*prev)->compound() == Compound::Type::NN ){
+	newt = Compound::Type::NNN;
+      }
+      BaseBracket *tmp = new BracketNest( CLEX::N, newt, debugFlag, myLog );
       tmp->append( *prev );
       tmp->append( *it );
       if ( debugFlag > 5 ){
-	cerr << "current result:" << parts << endl;
-	cerr << "new node:" << tmp << endl;
-	cerr << "erase " << *prev << endl;
+	LOG << "current result:" << parts << endl;
+	LOG << "new node:" << tmp << endl;
+	LOG << "erase " << *prev << endl;
       }
       prev = parts.erase(prev);
       if ( debugFlag > 5 ){
-	cerr << "erase " << *prev << endl;
+	LOG << "erase " << *prev << endl;
       }
       prev = parts.erase(prev);
       prev = parts.insert( prev, tmp );
       if ( debugFlag > 5 ){
-	cerr << "current result:" << parts << endl;
+	LOG << "current result:" << parts << endl;
       }
       it = prev;
       ++it;
@@ -937,7 +1038,94 @@ void BracketNest::resolveNouns( ){
     }
   }
   if ( debugFlag > 5 ){
-    cerr << "resolve NOUNS result:" << this << endl;
+    LOG << "resolve NOUNS result:" << this << endl;
+  }
+}
+
+list<BaseBracket*>::iterator BracketNest::glue( list<BaseBracket*>& result,
+						const list<BaseBracket*>::iterator& rpos ){
+  if ( debugFlag > 5 ){
+    LOG << "glue " << endl;
+    LOG << "result IN : " << result << endl;
+    LOG << "rpos= " << *rpos << endl;
+  }
+  size_t len = (*rpos)->RightHand.size();
+  bool matched = true;
+  vector<CLEX::Type> match_tags;
+  if ( len == 0 || len > result.size() ){
+    if ( debugFlag > 5 ){
+      LOG << "test MATCH FAIL (no RHS or RHS > result)" << endl;
+    }
+    matched = false;
+  }
+  else {
+    size_t j = 0;
+    list<BaseBracket*>::iterator it = rpos;
+    for (; j < len && it != result.end(); ++j, ++it ){
+      if ( debugFlag > 5 ){
+	LOG << "test MATCH vergelijk " << (*it)->tag() << " met " << (*rpos)->RightHand[j] << endl;
+      }
+      if ( (*rpos)->RightHand[j] == CLEX::GLUE ){
+	++j;
+	match_tags.push_back( (*rpos)->RightHand[j] );
+	continue; // the ^ is always OK
+      }
+      if ( (*rpos)->RightHand[j] != (*it)->tag() ){
+	if ( debugFlag > 5 ){
+	  LOG << "test MATCH FAIL (" << (*it)->tag()
+	       << " != " << (*rpos)->RightHand[j] << ")" << endl;
+	}
+	(*rpos)->set_status(Status::FAILED);
+	matched = false;
+      }
+      match_tags.push_back( (*rpos)->RightHand[j] );
+    }
+  }
+  if ( matched ){
+    list<BaseBracket*>::iterator bit = rpos;
+    if ( debugFlag > 5 ){
+      LOG << "OK een match" << endl;
+    }
+    list<BaseBracket*>::iterator it = bit--;
+    BracketNest *tmp
+      = new BracketNest( (*rpos)->tag(), Compound::Type::NONE, debugFlag, myLog );
+    for ( size_t j = 0; j < len-1; ++j ){
+      tmp->append( *it );
+      if ( debugFlag > 5 ){
+	LOG << "erase " << *it << endl;
+      }
+      it = result.erase(it);
+    }
+    if ( debugFlag > 5 ){
+      LOG << "new node:" << tmp << endl;
+      LOG << "match_tags = " << match_tags << endl;
+    }
+    tmp->_compound = construct( match_tags );
+    result.insert( ++bit, tmp );
+    return bit;
+  }
+  else {
+    // the glueing failed.
+    // we should try to start at the next node
+    list<BaseBracket*>::iterator bit = rpos;
+    return ++bit;
+  }
+}
+
+
+void BracketNest::resolveGlue( ){
+  list<BaseBracket*>::iterator it = parts.begin();
+  while ( it != parts.end() ){
+    // search for glue rules
+    if ( debugFlag > 5 ){
+      LOG << "search glue: bekijk: " << *it << endl;
+    }
+    if ( (*it)->isglue() ){
+      it = glue( parts, it );
+    }
+    else {
+      ++it;
+    }
   }
 }
 
@@ -946,11 +1134,11 @@ void BracketNest::resolveLead( ){
   while ( it != parts.end() ){
     // search for rules with a * at the begin
     if ( debugFlag > 5 ){
-      cerr << "search leading *: bekijk: " << *it << endl;
+      LOG << "search leading *: bekijk: " << *it << endl;
     }
     if ( (*it)->isNested() ){
       if ( debugFlag > 5 ){
-	cerr << "nested! " << endl;
+	LOG << "nested! " << endl;
       }
       (*it)->resolveLead();
       ++it;
@@ -971,11 +1159,11 @@ void BracketNest::resolveTail(){
   while ( it != parts.end() ){
     // search for rules with a * at the end
     if ( debugFlag > 5 ){
-      cerr << "search trailing *: bekijk: " << *it << endl;
+      LOG << "search trailing *: bekijk: " << *it << endl;
     }
     if ( (*it)->isNested() ){
       if ( debugFlag > 5 ){
-	cerr << "nested! " << endl;
+	LOG << "nested! " << endl;
       }
       (*it)->resolveTail();
       ++it;
@@ -985,9 +1173,9 @@ void BracketNest::resolveTail(){
       if ( (*it)->infixpos() > 0
 	   && (*it)->infixpos() == signed(len)-1 ){
 	if ( debugFlag > 5 ){
-	  cerr << "found trailing * " << *it << endl;
-	  cerr << "infixpos=" << (*it)->infixpos() << endl;
-	  cerr << "len=" << len << endl;
+	  LOG << "found trailing * " << *it << endl;
+	  LOG << "infixpos=" << (*it)->infixpos() << endl;
+	  LOG << "len=" << len << endl;
 	}
 	it = resolveAffix( parts, it );
       }
@@ -1003,11 +1191,11 @@ void BracketNest::resolveMiddle(){
   while ( it != parts.end() ){
     // now search for other rules with a * in the middle
     if ( debugFlag > 5 ){
-      cerr << "hoofd infix loop bekijk: " << *it << endl;
+      LOG << "hoofd infix loop bekijk: " << *it << endl;
     }
     if ( (*it)->isNested() ){
       if ( debugFlag > 5 ){
-	cerr << "nested! " << endl;
+	LOG << "nested! " << endl;
       }
       (*it)->resolveMiddle( );
       ++it;
@@ -1028,17 +1216,17 @@ void BracketNest::resolveMiddle(){
 void BracketNest::clearEmptyNodes(){
   // remove all nodes that don't have a morpheme or an inlection
   if ( debugFlag > 5 ){
-    cerr << "clear emptyNodes: " << this << endl;
+    LOG << "clear emptyNodes: " << this << endl;
   }
   list<BaseBracket*> out;
   list<BaseBracket*>::iterator it = parts.begin();
   while ( it != parts.end() ){
     if ( debugFlag > 5 ){
-      cerr << "loop clear emptyNodes : " << *it << endl;
+      LOG << "loop clear emptyNodes : " << *it << endl;
     }
     if ( (*it)->isNested() ){
       if ( debugFlag > 5 ){
-	cerr << "nested! " << endl;
+	LOG << "nested! " << endl;
       }
       (*it)->clearEmptyNodes( );
       out.push_back( *it );
@@ -1056,38 +1244,89 @@ void BracketNest::clearEmptyNodes(){
   }
   parts.swap( out );
   if ( debugFlag > 5 ){
-    cerr << "RESULT clear emptyNodes: " << this << endl;
+    LOG << "RESULT clear emptyNodes: " << this << endl;
   }
 }
 
 CLEX::Type BracketNest::getFinalTag() {
-  // cerr << "get Final Tag from: " << this << endl;
+  // LOG << "get Final Tag from: " << this << endl;
   CLEX::Type result_cls = CLEX::UNASS;
   auto it = parts.rbegin();
   while ( it != parts.rend() ){
-    //    cerr << "bekijk: " << *it << endl;
+    //    LOG << "bekijk: " << *it << endl;
     if ( (*it)->isNested()
 	 || ( (*it)->inflection().empty()
 	      && !(*it)->morpheme().isEmpty() ) ){
       result_cls = (*it)->tag();
-      //      cerr << "maybe tag = " << result_cls << endl;
+      //      LOG << "maybe tag = " << result_cls << endl;
       if ( result_cls != CLEX::P ){
+	// in case of P we hope for better to the left
 	auto it2 = it;
 	++it2;
 	if ( it2 != parts.rend() ){
-	  //	  cerr << "bekijk ook " << *it2 << endl;
+	  //	  LOG << "bekijk ook " << *it2 << endl;
 	  if ( (*it2)->infixpos() == 0 ){
 	    result_cls = (*it2)->tag();
+	    // in case of a X_*Y rule we need X
 	  }
 	}
-	// in case of P we hope for better
-	// in case of a X_*Y rule we need X
 	break;
       }
     }
+#ifdef NO_WAY
+    else if ( !(*it)->inflection().empty()
+	      && (*it)->morpheme().isEmpty() ){
+      string inf = (*it)->inflection();
+      // it is an inflection tag
+      LOG << " inflection: >" << inf << "<" << endl;
+      // given the specific selections of certain inflections,
+      //    select a tag!
+      CLEX::Type new_tag = CLEX::UNASS;
+      for ( size_t i=0; i < inf.size(); ++i ){
+	new_tag = CLEX::select_tag( inf[i] );
+	if ( new_tag != CLEX::UNASS ){
+	  LOG << inf[i] << " selects " << new_tag << endl;
+	  break;
+	}
+      }
+      if ( new_tag != CLEX::UNASS ) {
+	// apply the change. Remember, the idea is that an inflection is
+	// far more certain of the tag of its predecessing morpheme than
+	// the morpheme itself.
+	// This is not always the case, but it works
+	//
+	// go back to the previous morpheme
+	auto pit = it;
+	for( ++pit; pit != parts.rend(); ++pit ){
+	  CLEX::Type old_tag = (*pit)->tag();
+	  LOG << "een terug is " << *pit << endl;
+	  if ( CLEX::isBasicClass( old_tag ) &&
+	       old_tag != CLEX::P ){
+	    // only nodes that can get inflected (and unanalysed too)
+	    // now see if we can replace this class for a better one
+	    if ( old_tag == CLEX::PN && new_tag == CLEX::N ){
+	      LOG << "Don't replace PN by N" << endl;
+	    }
+	    else {
+	      LOG << " replace " << old_tag
+		  << " by " << new_tag << endl;
+	      (*pit)->setTag( new_tag );
+	      cls = new_tag;
+	    }
+	    return new_tag;
+	  }
+	}
+      }
+      else {
+	// this realy shouldn't happen. probably an error in the data!?
+	LOG << "inflection: " << inf
+	    << " Problem: DOESN'T select a tag" << endl;
+      }
+    }
+#endif
     ++it;
   }
-  //  cerr << "final tag = " << result_cls << endl;
+  //  LOG << "final tag = " << result_cls << endl;
   cls = result_cls;
   return cls;
 }
