@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2017
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -257,7 +257,7 @@ string find_class( unsigned int step,
 	result = classes[step];
       }
       else {
-	result = "0";
+	result = classes.back();
       }
     }
   }
@@ -309,17 +309,22 @@ void Mbma::clearAnalysis(){
 
 Rule* Mbma::matchRule( const std::vector<std::string>& ana,
 		       const UnicodeString& word ){
-  Rule *rule = new Rule( ana, word, mbmaLog, debugFlag );
+  Rule *rule = new Rule( ana, word, *mbmaLog, debugFlag );
   if ( rule->performEdits() ){
     rule->reduceZeroNodes();
     if ( debugFlag ){
       LOG << "after reduction: " << rule << endl;
     }
+#ifdef EXPERIMENT
+    rule->resolveBrackets( doDeepMorph );
+#endif
     rule->resolve_inflections();
     if ( debugFlag ){
       LOG << "after resolving: " << rule << endl;
     }
+#ifndef EXPERIMENT
     rule->resolveBrackets( doDeepMorph );
+#endif
     rule->getCleanInflect();
     if ( debugFlag ){
       LOG << "1 added Inflection: " << rule << endl;
@@ -371,7 +376,7 @@ void Mbma::addMorph( Word *word,
     }
     catch( const exception& e ){
       LOG << e.what() << " addMorph failed." << endl;
-      exit(EXIT_FAILURE);
+      throw;
     }
   }
   addMorph( ml, morphs );
@@ -420,7 +425,7 @@ void Mbma::addBracketMorph( Word *word,
     }
     catch( const exception& e ){
       LOG << e.what() << " addBracketMorph failed." << endl;
-      exit(EXIT_FAILURE);
+      throw;
     }
   }
   args["class"] = "stem";
@@ -456,6 +461,9 @@ void Mbma::addBracketMorph( Word *word,
 void Mbma::addBracketMorph( Word *word,
 			    const string& orig_word,
 			    const BracketNest *brackets ) const {
+  if (debugFlag){
+    LOG << "addBracketMorph(" << word << "," << orig_word << "," << brackets << ")" << endl;
+  }
   KWargs args;
   args["set"] = mbma_tagset;
   MorphologyLayer *ml;
@@ -466,7 +474,7 @@ void Mbma::addBracketMorph( Word *word,
     }
     catch( const exception& e ){
       LOG << e.what() << " addBracketMorph failed." << endl;
-      exit(EXIT_FAILURE);
+      throw;
     }
   }
   Morpheme *m = 0;
@@ -475,7 +483,7 @@ void Mbma::addBracketMorph( Word *word,
   }
   catch( const exception& e ){
     cerr << "createMorpheme failed: " << e.what() << endl;
-    exit(1);
+    throw;
   }
   if ( m ){
     args.clear();
@@ -626,8 +634,9 @@ void Mbma::filterSubTags( const vector<string>& feats ){
 	}
       }
     }
-    if (debugFlag)
+    if (debugFlag){
       LOG << "score: " << match_count << " max was " << max_count << endl;
+    }
     if (match_count >= max_count) {
       if (match_count > max_count) {
 	max_count = match_count;
@@ -701,7 +710,7 @@ void Mbma::filterSubTags( const vector<string>& feats ){
     int i=0;
     for ( const auto& it : analysis ){
       LOG << ++i << " - " << it << " " << it->getKey(false)
-		    << " (" << it->getKey(false).length() << ")" << endl;
+	  << " (" << it->getKey(false).length() << ")" << endl;
     }
     LOG << "" << endl;
   }
@@ -722,6 +731,12 @@ void Mbma::filterSubTags( const vector<string>& feats ){
   return;
 }
 
+void Mbma::assign_compounds(){
+  for ( auto const& sit : analysis ){
+    sit->compound = sit->brackets->getCompoundType();
+  }
+}
+
 void Mbma::getFoLiAResult( Word *fword, const UnicodeString& uword ) const {
   if ( analysis.size() == 0 ){
     // fallback option: use the word and pretend it's a morpheme ;-)
@@ -826,6 +841,7 @@ void Mbma::Classify( Word* sword ){
     }
     filterHeadTag( head );
     filterSubTags( featVals );
+    assign_compounds();
     getFoLiAResult( sword, lWord );
   }
 }
@@ -866,3 +882,19 @@ vector<string> Mbma::getResult() const {
   }
   return result;
 }
+
+vector<pair<string,string>> Mbma::getResults( ) const {
+  vector<pair<string,string>> result;
+  for ( const auto& it : analysis ){
+    string tmp = it->morpheme_string( true );
+    string cmp = toString( it->compound );
+    result.push_back( make_pair(tmp,cmp) );
+  }
+  if ( debugFlag ){
+    LOG << "result of morph analyses: ";
+    for ( const auto& r : result ){
+      LOG << " " << r.first << "/" << r.second << "," << endl;
+    }
+  }
+  return result;
+}
